﻿B4A=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=9.85
@EndOfDesignText@
#Region Project Notes
'DMX Client to set the color of an LED PAR (DMX512 Mode) via BLE.
'Usage:
'Step 1: Connect to the DMX BLE Controller (ESP32 with B4R program) first.
'Step 2: Set dimmer level
'Step 3: Set any color
'
'Concept
'An external Python script, located in the same folder as the dmxclient.jar handles BLE communication with the ESP32.
'Read the source of bleinterface.py.
#End Region

#Region Shared Files
#CustomBuildAction: folders ready, %WINDIR%\System32\Robocopy.exe,"..\..\Shared Files" "..\Files"
'Ctrl + click to sync files: ide://run?file=%WINDIR%\System32\Robocopy.exe&args=..\..\Shared+Files&args=..\Files&FilesSync=True
'Ctrl + click to export as zip: ide://run?File=%B4X%\Zipper.jar&Args=Project.zip
'See the list of page related events in the B4XPagesManager object. The event name is B4XPage.
#End Region

Sub Class_Globals
	
	'Constants
	Private Const APP_TITLE As String = "DMX Client"
	Private Const APP_VERSION As String = "200240612"
	Private Const APP_INFO As String = "Example controlling an LED PAR set to DMX512 Mode via BLE."
	Private Const APP_HINT As String = "Ensure to set the dimmer first after connecting to the DMX BLE Controller (ESP32)."

	'Option to see the debug log from the esp32
	Private Const APP_DEBUG As Boolean = False

	'UI	
	Private Root As B4XView
	Private xui As XUI
	Private btnConnect As B4XView
	Private lblConnectStatus As B4XView
	Private btnSetChannelValue As B4XView
	Private spAddress As Spinner
	Private SliderLevelBlue As SliderLevel
	Private SliderLevelDimmer As SliderLevel
	Private SliderLevelGreen As SliderLevel
	Private SliderLevelRed As SliderLevel
	Private spChannel As Spinner
	Private tfDMXInterface As B4XView
	Private tfValue As B4XView
	Private lvLog As ListView
	Private blnClearLog As B4XView
	Private lblInfo As B4XView
	
	'Communication
	'Shell object used for running the python script as process
	Private shl As Shell
	
	'BLE - script is located in same folder as the jar
	Private BLECOMMANDHANDLERSCRIPT As String = "dmxbleinterface.py"
	Private isConnected As Boolean = False
 
	'LED address, channel range 0-512, level range 0-100, value range 0-255 (sent to the DMX interface)
	Private LED_Address As Int = 1
	Private Const CHANNEL_MIN As Int = 1
	Private Const CHANNEL_MAX As Int = 512
	Private Const LEVEL_MIN As Int = 0
	Private Const LEVEL_MAX As Int = 100
	Private Const VALUE_MIN As Int = 0
	Private Const VALUE_MAX As Int = 255
End Sub

Public Sub Initialize
	B4XPages.GetManager.LogEvents = True
End Sub

'This event will be called once, before the page becomes visible.
Private Sub B4XPage_Created (Root1 As B4XView)
	Root = Root1
	Root.LoadLayout("MainPage")
	
	'Set mainpage properties
	B4XPages.SetTitle(Me, $"${APP_TITLE} - ${APP_VERSION}"$)
	B4XPages.GetNativeParent(Me).Resizable = False    
	
	'Set defaults
	lblConnectStatus.Text = "Disconnected"
	lblInfo.Text = $"${APP_INFO}${CRLF}${APP_HINT}"$
	
	spAddress.Value = LED_Address

	CallSubDelayed(Me, "Set_SliderLevel_Properties")
End Sub

'NOT USED = just to show how to use GetView 0=LabelText,1=SliderLevel,2=LabelLevel
Private Sub Set_SliderLevel_Properties
'	Dim LabelText As B4XView = SliderLevelDimmer.mBase.GetView(0)
'	LabelText.TextColor = xui.Color_Red
End Sub

Private Sub btnConnect_Click
	If Not(isConnected) Then
		RunBLECommandHandler
	Else
		shl_KillProcess
	End If
End Sub

Private Sub spAddress_ValueChanged (Value As Object)
	LED_Address = Value
End Sub

Private Sub SliderLevelDimmer_ValueChange (Value As Double)
	Set_DMX_Channel_Value(LED_Address, Value)
End Sub

Private Sub SliderLevelRed_ValueChange (Value As Double)
	Set_DMX_Channel_Value(LED_Address + 1, Value)
End Sub

Private Sub SliderLevelGreen_ValueChange (Value As Double)
	Set_DMX_Channel_Value(LED_Address + 2, Value)
End Sub

Private Sub SliderLevelBlue_ValueChange (Value As Double)
	Set_DMX_Channel_Value(LED_Address + 3, Value)
End Sub

Private Sub spChannel_ValueChanged (Value As Object)
	Log($"spChannel_value_Changed: ${Value}"$)
End Sub

Private Sub tfValue_Action
	btnSetChannelValue_Click
End Sub

Private Sub btnSetChannelValue_Click
	Set_DMX_Channel_Value(spChannel.Value, tfValue.Text.As(Int))
End Sub

Private Sub blnClearLog_Click
	lvLog.Items.Clear	
End Sub

#Region DMX
'Set DMX channel and value by sending async write to the DMX RESTFul server.
Sub Set_DMX_Channel_Value(channel As Int, value As Int)
	'Do nothing if not connected to the DMX RESTFul server
	If Not(isConnected) Then Return

	'Map from level 0-100 to value 0-255
	value = MapRange(value, LEVEL_MIN, LEVEL_MAX, VALUE_MIN, VALUE_MAX)
	
	'Check if the value is in range and write the command. 
	'The command is an HTTP GET command with arguments.
	If (channel >= CHANNEL_MIN And channel <= CHANNEL_MAX) And (value >= VALUE_MIN And value <= VALUE_MAX) Then
		shl_WriteCommand($"${channel},${value}"$)
		ToLog($"[Set_Channel_Value]Set channel=${channel},value=${value}"$)
	Else
		ToLog($"[Set_Channel_Value][ERROR]Can not set channel ${channel} or value ${value} for the DMX Controller - Check range"$)
	End If
End Sub
#End Region

#Region SHELL
'Windows / Linux Command Handler - BLE interface between client (with no BLE) & the BLE server.

'Run the python script dmxinterface.py located in the same folder as the jar.
'The python script listens for commands and sends the command result back to stderr.
'When the script starts, it tries to connect to the peripheral as defined in the script.
'Note that this will take some time - watch the log.
Private Sub RunBLECommandHandler
	ToLog($"[RunBLECommandHandler]Python Script starting ${BLECOMMANDHANDLERSCRIPT}"$)

	'Init the shell with python and the python script as parameter.
	If APP_DEBUG Then
		shl.Initialize("shl", "python", Array As String($"${BLECOMMANDHANDLERSCRIPT}"$, "-d 1"))
	Else
		shl.Initialize("shl", "python", Array As String($"${BLECOMMANDHANDLERSCRIPT}"$))
	End If

	'Enable input to stop the python process
	shl.InputStreamEnabled = True

	'Run the python script as process and handle the output event in shl_StdOut & StdErr.
	shl.RunWithOutputEvents(-1)

	lblConnectStatus.Text = "Connecting..."
	ToLog("[RunBLECommandHandler]Connecting to the DMX Interface")
End Sub

'Sub shl_ProcessCompleted (Success As Boolean, ExitCode As Int, StdOut As String, StdErr As String)
'	Log("Success:" & Success)
'	Log("StdErr:" & StdErr)
'	Log("StdOut:" & StdOut)
'End Sub

'NOTE on StdOut & StdErr:
'StdOut contains simpleble and script debug info.
'StdErr contains the status & message info without debug info.

'Handle stdout from the python script.
'The info contains debug information.
Sub shl_StdOut(Buffer() As Byte, Length As Int)
	Sleep(100)
	CallSub2(Me, "ToLog", $"[shl_StdOut]${BytesToString(Buffer, 0, Length, "utf8")}"$)
End Sub

'Handle stderr from the python script.
'The info contains connection, error and status info.
Sub shl_StdErr(Buffer() As Byte, Length As Int)
	'ToLog($"shl_StdErr: ${BytesToString(Buffer, 0, Length, "utf8")}"$)
	'Get the data from the stdout string
	Dim data() As String = Regex.Split(",", BytesToString(Buffer, 0, Length, "utf8"))

	'Two objects expected
	If data.Length == 2 Then
		Dim status As String = data(0)
		Dim msg As String = data(1)
		'ToLog($"shl_StdErr: status=${status},msg=${msg}"$)
		
		'To update the user interface need to have a short sleep to then call a sub callsub to update the UI view ledstate
		'Because the shl process runs in the background
		Sleep(100)
		CallSub3(Me, "SetState", status, msg)
	End If
End Sub

'Write a command to the inputstream read by the python script.
'Do nothing if not connected. Requires global flag isConnected (boolean).
Sub shl_WriteCommand(command As String)
	Dim msg As String
	ToLog($"[shl_WriteCommand]cmd=${command}, isconnected=${isConnected}"$)

	'Leave if not connected
	If Not(isConnected) Then
		msg = $"[shl_WriteCommand][WARNING]Not connected to the DMX Interface"$
		Sleep(100)
		CallSub3(Me, "SetState", msg, "")
		Return
	End If
	
	'Add crlf to the command string
	Dim cmd As String = $"${command}${CRLF}"$
	Try
		shl.WriteToInputStream(cmd.GetBytes("UTF8"))
	Catch
		'There is an error - disconnect
		isConnected = False
		msg = $"[shl_WriteCommand][ERROR]cmd=${command}, lastexception=${LastException}"$
		Sleep(100)
		CallSub3(Me, "SetState", msg, "")
	End Try
End Sub

'Handle the state which is a JSON object. Examples:
Sub SetState(state As String, msg As String)
	ToLog($"[SetState]state=${state}, msg=${msg.Trim}"$)
	'Error returned from the dmxinterface
	If state == "0" Then
		isConnected = False
		btnConnect.Text = "Connect"
		lblConnectStatus.Text = "Disconnected"
	End If
	'Connected returned from the dmxinterface
	If state == "1" Then
		'Set connection flag by checking status is 1 (as defined in the Python script)
		isConnected = True
		btnConnect.Text = "Disconnect"
		lblConnectStatus.Text = "Connected"
	End If
End Sub

'Kill the shell process
Sub shl_KillProcess
	Dim msg As String
	Try
		shl.KillProcess
		msg = $"[shl_KillProcess]Done"$
	Catch
		msg = $"[shl_KillProcess][ERROR]${LastException}"$
	End Try
	isConnected = False
	btnConnect.Text = "Connect"
	lblConnectStatus.Text = "Disconnected"
	ToLog(msg)
End Sub
#End Region

#Region Helpers
Private Sub MapRange(Value As Int, FromLow As Int, FromHigh As Int, ToLow As Int, ToHigh As Int) As Int
	Return ToLow + (ToHigh - ToLow) * ((Value - FromLow) / (FromHigh - FromLow))
End Sub

Private Sub ToLog(msg As String)
	msg = $"${DateTime.Time(DateTime.Now)} ${msg}"$
	lvLog.Items.InsertAt(0, msg)
	Log(msg)
End Sub
#End Region
