﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=10
@EndOfDesignText@
'File: Slider.bas
'Date: 20240612
'Author: Robert W.B. Linn
'CustomView Slider for selecting a level between 0 - 100 (default).
'Depends on Libraries: JavaObject

'Ref: https://www.b4x.com/android/forum/threads/b4x-custom-views-with-enhanced-designer-support.62488/

'ChangeLog:
'20240611: [NEW] OnMouseReleased to change value only if the mouse is released (set property valuechanging to false)

'Events
'Handle slider value change
#Event: ValueChange (Value As Double)

'Set the designer properties for the text, slider and value indicator
'The keys must in lowercase 
#DesignerProperty: Key: labeltext, Displayname: Label Text, Fieldtype: String, Defaultvalue: Level:, Description: The label to show the text
#DesignerProperty: Key: labellevel, Displayname: Label Level, Fieldtype: String, Defaultvalue: 0, Description: The label to show the slider level
#DesignerProperty: Key: level, Displayname: Level, Fieldtype: Int, , MinRange: 0, MaxRange: 100, Defaultvalue: 0, Description: The slider level 0-100
#DesignerProperty: Key: minlevel, Displayname: Min Level, Fieldtype: Int, MinRange: 0, MaxRange: 100, Defaultvalue: 0, Description: The slider level min value (default 0)
#DesignerProperty: Key: maxlevel, Displayname: Max Level, Fieldtype: Int, MinRange: 0, MaxRange: 100, Defaultvalue: 100, Description: The slider level max value (default 100)
#DesignerProperty: Key: showlevel, Displayname: Show Level, Fieldtype: Boolean, Defaultvalue: False, Description: The option to show the value level (default false)
#DesignerProperty: Key: roundlevel, Displayname: Round Level, Fieldtype: Boolean, Defaultvalue: True, Description: The option to round the value level from float to int (default true)
#DesignerProperty: Key: blockincrement, Displayname: Block Increment, Fieldtype: Float, Defaultvalue: 10.0, Description: The slider block increment (default 10.0)
#DesignerProperty: Key: majortickunit, Displayname: Major Tick Unit, Fieldtype: Float, Defaultvalue: 10.0, Description: The slider major tick unit (default 20.0)
#DesignerProperty: Key: minortickcount, Displayname: Minor Tick Count, Fieldtype: Int, Defaultvalue: 20, Description: The slider minor tick count (default 20)
#DesignerProperty: Key: ticklabels, Displayname: Tick Labels, Fieldtype: Boolean, Defaultvalue: True, Description: The option to show the tick labels (default true)
#DesignerProperty: Key: tickmarks, Displayname: Tick Marks, Fieldtype: Boolean, Defaultvalue: True, Description: The option to show the tick marks (default true)
#DesignerProperty: Key: valuechanging, Displayname: Value Changing, Fieldtype: Boolean, Defaultvalue: True, Description: The option to show the value changing (default true)
#DesignerProperty: Key: snaptoticks, Displayname: Snap to Ticks, Fieldtype: Boolean, Defaultvalue: True, Description: The option to snap the value to the ticks (default true)
#DesignerProperty: Key: thumbcolor, Displayname: Thumb Color, Fieldtype: Color, Defaultvalue: 0xFFCFDCDC, Description: The option to set the thumb color

Sub Class_Globals
	'Events
	Private mEventName As String 'ignore
	Private mCallBack As Object 'ignore

	'Base
	Public mBase As B4XView
	Private mProps As Map 'ignore
	Private xui As XUI 'ignore
	Public Tag As Object
	Private mLbl As Label
	
	'Views
	Private LabelText As B4XView
	Private SldrLevel As Slider
	Private LabelLevel As B4XView
	
	'Properties
	Private mLabelText As String
	Private mLabelTextBold As Boolean
	Private mLabelLevel As String
	Private mLevel As Int
	Private mMinLevel As Int
	Private mMaxLevel As Int
	Private mShowLevel As Boolean
	Private mRoundLevel As Boolean
	Private mBlockIncrement As Double
	Private mMajorTickUnit As Double
	Private mMinorTickCount As Int
	Private mTickLabels As Boolean
	Private mTickMarks As Boolean
	Private mValueChanging As Boolean
	Private mSnapToTicks As Boolean
	Private mThumbColor As Paint

	'Slider as JavaObject used to set properties
	Private joSlider As JavaObject
End Sub

Public Sub Initialize (Callback As Object, EventName As String)
	mEventName = EventName
	mCallBack = Callback
	'Log($"${mEventName}=${mCallBack}"$)
End Sub

'Base type must be Object
Public Sub DesignerCreateView (Base As Object, Lbl As Label, Props As Map)
	mBase = Base
    Tag = mBase.Tag
	mBase.Tag = Me
	mLbl = Lbl
	mProps = Props

	'Dim clr As Int = xui.PaintOrColorToColor(Props.Get("TextColor")) 'Example of getting a color value from Props

	'Load the layout file using CallSubDelayed
	CallSubDelayed2(Me, "AfterLoadLayout", Props)
End Sub

Private Sub Base_Resize (Width As Double, Height As Double)
  
End Sub

Sub AfterLoadLayout(Props As Map)
	'Load the layout created with the visual designer
	mBase.LoadLayout("SliderLevel")

	'Set properties
	
	'For the labels following CV label setting can be used:
	'TextColor, TextSize, Font
	LabelText.TextColor = xui.PaintOrColorToColor(mLbl.TextColor)
	LabelText.TextSize = mLbl.TextSize
	LabelText.Font = mLbl.Font
	LabelLevel.Font = mLbl.Font
	LabelLevel.TextColor = xui.PaintOrColorToColor(mLbl.TextColor)
	LabelLevel.TextSize = mLbl.TextSize
	
	'Label Text
	setLabelText(Props.Get("labeltext"))

	'Slider = Assign a JavaObject as Slider
	joSlider = SldrLevel

	'Set level & min/max
	setLevel(Props.Get("level"))
	setMinLevel(Props.Get("minlevel"))
	setMaxLevel(Props.Get("maxlevel"))
	setRoundLevel(Props.Get("roundlevel"))

	'Set additional properties which are not provided by the visual designer
	setBlockIncrement(Props.Get("blockincrement"))
	setMajorTickUnit(Props.Get("majortickunit"))
	setMinorTickCount(Props.Get("minortickcount"))
	setTickLabels(Props.Get("ticklabels"))
	setTickMarks(Props.Get("tickmarks"))
	setValueChanging(Props.Get("valuechanging"))
	setSnapToTicks(Props.Get("snaptoticks"))

	'ThumColor is HEX value, like 0x0000FFFF for Blue
	'Need short sleep for the UI to complete loading the layout
	Sleep(1)
	setThumbColor(Props.Get("thumbcolor"))
	
	'Set the Label Value and itis visibility
	setShowLevel(Props.Get("showlevel"))
	setLabelLevel(Props.Get("labellevel"))
	
	'Set onMouseRelease Event to trigger value change only if mouse released
	Dim oSlider As Object = joSlider.CreateEvent("javafx.event.EventHandler","SliderMouseReleased",False)
	joSlider.RunMethod("setOnMouseReleased",Array(oSlider))
End Sub

'Properties
'Set/Get the label text
Public Sub setLabelText(Text As String)
	mLabelText = Text
	LabelText.Text = mLabelText
End Sub
Public Sub getLabelText As String
	Return mLabelText
End Sub

'Set/Get the label text attribute bold
Public Sub setLabelTextBold(State As Boolean)
	mLabelTextBold = State
	If State Then
		CSSUtils.SetStyleProperty(LabelText, "-fx-font-weight", "bold")
	Else
		CSSUtils.SetStyleProperty(LabelText, "-fx-font-weight", "normal")
	End If
End Sub
Public Sub getLabelTextBold As Boolean
	Return mLabelTextBold
End Sub

'Set/Get the level
Public Sub setLevel(Value As Int)
	'Check if the level is in range min/max level
	If (Value >= mMinLevel And Value <= mMaxLevel) Then
		mLevel = Value
		SldrLevel.Value = mLevel
	End If
End Sub
Public Sub getLevel As Int
	Return mLevel
End Sub

'Set/Get the min level
Public Sub setMinLevel(Value As Int)
	mMinLevel = Value
	SldrLevel.MinValue = mMinLevel
End Sub
Public Sub getMinLevel As Int
	Return mMinLevel
End Sub

'Set/Get the max level
Public Sub setMaxLevel(Value As Int)
	mMaxLevel = Value
	SldrLevel.MaxValue = mMaxLevel
End Sub
Public Sub getMaxLevel As Int
	Return mMaxLevel
End Sub

'Set/Get the label level
Public Sub setLabelLevel(Text As String)
	mLabelLevel = Text
	LabelLevel.Text = mLabelLevel
End Sub
Public Sub getLabelLevel As String
	Return mLabelLevel
End Sub

'Set/Get option show label level
Public Sub setShowLevel(State As Boolean)
	mShowLevel = State
	LabelLevel.Visible = mShowLevel
End Sub
Public Sub getShowLevel As Boolean
	Return mShowLevel
End Sub

'Set/Get option round level
Public Sub setRoundLevel(State As Boolean)
	mRoundLevel = State
End Sub
Public Sub getRoundLevel As Boolean
	Return mRoundLevel
End Sub

'Set/Get option block increment - Must be double
Public Sub setBlockIncrement(Value As Double)
	mBlockIncrement = Value
	joSlider.RunMethod("setBlockIncrement", Array(mBlockIncrement))
End Sub
Public Sub getBlockIncrement As Double
	Return mBlockIncrement
End Sub

'Set/Get option major tick unit - Must be double
Public Sub setMajorTickUnit(Value As Double)
	mMajorTickUnit = Value
	joSlider.RunMethod("setMajorTickUnit", Array(mMajorTickUnit))
End Sub
Public Sub getMajorTickUnit As Double
	Return mMajorTickUnit
End Sub

'Set/Get option major tick count - Must be int
Public Sub setMinorTickCount(Value As Int)
	mMinorTickCount = Value
	joSlider.RunMethod("setMinorTickCount", Array(Value))
End Sub
Public Sub getMinorTickCount  As Int
	Return mMinorTickCount 
End Sub

'Set/Get option tick labels
Public Sub setTickLabels(State As Boolean)
	mTickLabels = State
	joSlider.RunMethod("setShowTickLabels", Array(State))
End Sub
Public Sub getTickLabels As Boolean
	Return mTickLabels
End Sub

'Set/Get option tick marks
Public Sub setTickMarks(State As Boolean)
	mTickMarks = State
	joSlider.RunMethod("setShowTickMarks", Array(State))
End Sub
Public Sub getTickMarks As Boolean
	Return mTickMarks
End Sub

'Set/Get option value changing
Public Sub setValueChanging(State As Boolean)
	mValueChanging = State
	joSlider.RunMethod("setValueChanging", Array(State))
End Sub
Public Sub getValueChanging As Boolean
	Return mValueChanging
End Sub

'Set/Get option snap to ticks
Public Sub setSnapToTicks(State As Boolean)
	mSnapToTicks = State
	joSlider.RunMethod("setSnapToTicks", Array(State))
End Sub
Public Sub getSnapToTicks As Boolean
	Return mSnapToTicks
End Sub

'Set/Get option thumb color
Public Sub setThumbColor(Clr As Paint)
	mThumbColor = Clr
	Dim c() As Int = GetRGB(Clr)
	Dim r As Int = c(0)
	Dim g As Int = c(1)
	Dim b As Int = c(2)
	joSlider.RunMethodJo("lookup", Array(".thumb")).RunMethod("setStyle", Array($"-fx-background-color:rgb(${r},${g},${b});"$))
End Sub
Public Sub getThumbColor As Paint
	Return mThumbColor
End Sub

#Region Events
'Event: Slider value change
'Handle slider mousereleased event.
'The mouse event API method getSource is used to get the slider clicked. The event first argument is used for that.
'Then get the slider value to call then ValueChange event.
Private Sub SliderMouseReleased_Event(EventName As String, Args() As Object) As Object  'ignore
	'Only use if the valuechanging proprty is not used (set in designer).
	If Not(mValueChanging) Then
		If Args.length > 0 Then
			Dim KEventJO As JavaObject = Args(0)
			Dim sldr As Slider = KEventJO.RunMethod("getSource",Null)
			SldrLevelValueChange(sldr.Value)
		End If		
	End If
	Return False
End Sub

'Slider value change called by event MouseReleased or event ValueChange.
Private Sub SldrLevelValueChange(Value As Double)
	'Check if the value should be rounded
	If mRoundLevel Then
		Value = Round(Value)
	End If

	'Update the labellevel if show flag is set
	If mShowLevel Then
		setLabelLevel(Value)
	End If

	'Check if the callback sub exists on the callback
	If SubExists(mCallBack, mEventName & "_ValueChange") Then
		CallSub2(mCallBack, mEventName & "_ValueChange", Value)
	End If
End Sub

'Event: Slider value change
Private Sub SldrLevel_ValueChange(Value As Double)
	If mValueChanging Then
		SldrLevelValueChange(Value)
	End If
End Sub
#End Region

'*********************************************
'HELPERS
'*********************************************
#Region Helpers
'Get the R-G-B values 0-255 from ColorValue HEX format.
'Example 0x0000FFFF for Blue returns 0,0,255
Private Sub GetRGB(ColorValue As Paint) As Int()
	'Define the result array with 3 items
	Dim result(3) As Int
	Dim joCV As JavaObject = ColorValue
	'Get the color values as double between 0-1
	Dim R As Double = joCV.RunMethod("getRed", Null)
	Dim G As Double = joCV.RunMethod("getGreen", Null)
	Dim B As Double = joCV.RunMethod("getBlue", Null)
	'Assign the color value to a value 0-255 rounded
	result(0) = Round(R * 255)
	result(1) = Round(G * 255)
	result(2) = Round(B * 255)
	Return result
End Sub

'Private Sub MapRange(Value As Int, FromLow As Int, FromHigh As Int, ToLow As Int, ToHigh As Int) As Int
'	Return ToLow + (ToHigh - ToLow) * ((Value - FromLow) / (FromHigh - FromLow))
'End Sub
#End Region
