# B4R Library rConceptinetics - README Example DMX Control Bluetooth Low Energy

## Purpose
To control DMX LED lights from a desktop application sending commands via Bluetooth Low Energy (BLE).

## Concept
An ESP32-WROOM-32 (ESP32) runs a B4R program acting as the DMX BLE Controller (with identifier DMXBLECONTROLLER).
The ESP32 waits for BLE client connections and listens to BLE payload received.
The ESP32 is connected via its Serial2 port to an Arduino MEGA Serial2 port.
The ESP32 parses the payload received into 2 objects channel,value and sends the objects serialized to the Arduino MEGA.
The Arduino MEGA + Arduino DMX shield is the DMX Controller to set the LED channel and value (brightness) via an B4R program.

**Data Objects**
* channel=1-512 (mandatory)
* value=0-255 (mandatory)

## Hardware
* 1x Arduino MEGA
* 1x CQRobot DMX Shield MAX485 for MCU
* 1x ESP-32S USB C Developmentboard (ESP32)
* 1x LED Mini Flat Par Light LSPA36RC (DMX512 with 3-7 channels)
* 1x Adam Hall Cables 3 STAR DMF 0600 DMX Cable 3-Pol XLR Female auf 3-Pol XLR male 6 m 
* 1x Adam Hall Connectors 3 STAR DMX T 3 XLR Cable 3-Pol woth 120OHM resistor

## Software
* B4R 4.0 - DMXBLEController ESP32
* B4R 4.0 - DMXController Arduino MEGA
* B4R Additional Library rConceptinetics 0.5
* B4J 10.0 - DMXClient UI with CustomView SliderLevel to control a DMX channel value
* Python 3.10.11, Thonny 4.1.4 - Used by the B4J DMXClient

## Wiring
**ESP32 = Arduino MEGA**
```
Pin RX2 GPIO16 = TX2 GPIO16 (GREEN)
Pin TX2 GPIO17 = RX2 GPIO17 (BLUE)
For both controllers the onboard serial2 ports are used.
```

## Power Supply
On the ESP32S Developmentboard, the jumper 3.3V-5V is changed from 3.3V to 5V.
The power is supplied to the ESP32S Developmentboard either USB or external 9V.
**ESP32 = Arduino MEGA**
```
Pin 5V = VIN (RED)
Pin GND = GND (BLACK)
```

**Arduino MEGA = DMX Shield**
```
Pin 18 TX1 = 3 RX (Blue)
Pin 19 RX1 = 4 TX (Green)
```

## Clients
The BLE connection has been tested with the Android App Serial Bluetooth Terminal 1.47.
After selecting the device with identifier DMXBLECONTROLLER, send string 1,100 followed by 2,100 to check if LED lights RED.

A B4J UI test application, DMX Client, has been created based on the DMXRESTServerWiFi example.
This DMX client uses an external Python script (dmxbleinterface.py) to handle BLE communication with the DMX BLE Controller (ESP32).
The Python script is located in the same folder as the dmxclient.jar.
The data flow:
B4J DMX Client > Python Script BLE Connection to DMXBLEController (ESP32)
B4J DMX Client > set channel,value > send channel,value > Python Script BLE data handling > DMXBLEController
DMXBLEController > parse string channel,value > serialize data > DMXController (Arduino MEGA)
DMXController > deserialize data > set DMX channel & value

Any BLE client can be used.

## Notes
* For more information, see README.md of the rConceptinetics library.
* This BLE concept could also be used for other applications.

## ToDo
* Develop B4A DMX BLE client